import { defineStore } from "pinia";
import { BucketPermissionService, BucketService } from "@/client/s3proxy";
import type {
  BucketOut,
  BucketIn,
  BucketPermissionOut,
} from "@/client/s3proxy";
import { useAuthStore } from "@/stores/auth";

export const useBucketStore = defineStore({
  id: "buckets",
  state: () =>
    ({
      buckets: [],
      ownPermissions: {},
    } as {
      buckets: BucketOut[];
      ownPermissions: Record<string, BucketPermissionOut>;
    }),
  getters: {
    writableBuckets(): BucketOut[] {
      return this.buckets.filter(
        (bucket) =>
          this.ownPermissions[bucket.name] === undefined ||
          this.ownPermissions[bucket.name].permission !== "READ"
      );
    },
    getBucketPermission(): (
      bucketName: string
    ) => BucketPermissionOut | undefined {
      return (bucketName) => this.ownPermissions[bucketName];
    },
    writableBucket(): (bucketName: string) => boolean {
      return (bucketName) =>
        this.ownPermissions[bucketName] === undefined ||
        this.ownPermissions[bucketName].permission !== "READ";
    },
    readableBucket(): (bucketName: string) => boolean {
      return (bucketName) =>
        this.ownPermissions[bucketName] === undefined ||
        this.ownPermissions[bucketName].permission !== "WRITE";
    },
  },
  actions: {
    _fetchOwnPermissions(
      onFulfilled:
        | ((buckets: BucketPermissionOut[]) => void)
        | null
        | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      const authStore = useAuthStore();
      if (authStore.user != null) {
        BucketPermissionService.bucketPermissionListPermissionsPerUser(
          authStore.user.uid
        )
          .then((permissions) => {
            const new_permissions: Record<string, BucketPermissionOut> = {};
            for (const perm of permissions) {
              new_permissions[perm.bucket_name] = perm;
            }
            this.ownPermissions = new_permissions;
            onFulfilled?.(permissions);
          })
          .catch(onRejected)
          .finally(onFinally);
      }
    },
    deleteOwnPermission(bucketName: string) {
      this.buckets = this.buckets.filter(
        (bucket) => bucket.name !== bucketName
      );
      delete this.ownPermissions[bucketName];
    },
    fetchBuckets(
      onFulfilled: ((buckets: BucketOut[]) => void) | null | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      BucketService.bucketListBuckets()
        .then((buckets) => {
          this.buckets = buckets;
          onFulfilled?.(buckets);
        })
        .catch(onRejected)
        .finally(onFinally);
      this._fetchOwnPermissions();
    },
    fetchBucket(
      bucketName: string,
      onFulfilled: ((bucket: BucketOut) => void) | null | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      BucketService.bucketGetBucket(bucketName)
        .then((bucket) => {
          this.buckets[
            this.buckets.findIndex(
              (tempBucket) => tempBucket.name === bucket.name
            )
          ] = bucket;
          onFulfilled?.(bucket);
        })
        .catch(onRejected)
        .finally(onFinally);
    },
    deleteBucket(
      bucketName: string,
      onFulfilled: (() => void) | null | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      BucketService.bucketDeleteBucket(bucketName, true)
        .then(() => {
          this.buckets = this.buckets.filter(
            (bucket) => bucket.name !== bucketName
          );
          onFulfilled?.();
        })
        .catch(onRejected)
        .finally(onFinally);
    },
    createBucket(
      bucket: BucketIn,
      onFulfilled:
        | ((createdBucket: BucketOut) => void)
        | null
        | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      BucketService.bucketCreateBucket(bucket)
        .then((createdBucket) => {
          this.buckets.push(createdBucket);
          this.buckets.sort((bucketA, bucketB) =>
            bucketA.name >= bucketB.name ? 1 : -1
          );
          onFulfilled?.(createdBucket);
        })
        .catch(onRejected)
        .finally(onFinally);
    },
  },
});
